/*
 Open Bee Hive Scale Wireless GET-Sketch
 2015-01 by mois
 Dieser Sketch von mois ist lizensiert unter einer Creative Commons Namensnennung - Nicht-kommerziell - Weitergabe unter gleichen Bedingungen 4.0 International Lizenz.
 This code by mois is licensed under a Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International License.
 */ 
 
// load cell characteristic
// define here individual values for used load cell type
//
// - 11994383 sensor value for "0 kg"
//   write down the sensor value of the scale with no load and adjust it here
// - 6016676 is the value for a 1 kg weight load
//   add a load with known weight in kg to the ceall, note the sesor value, calculate
//   the value for a 1 kg load and adjust it here
long loadCellZeroOffset = 6800000;
long loadCellKgDivider = 6016676;
#include <RedFly.h>
#include <RedFlyClient.h>

byte ip[]        = { 192,168,178, 29 }; //ip from shield (client)
byte netmask[]   = { 255,255,255,  0 }; //netmask
byte gateway[]   = { 192,168,178,1 }; //ip from gateway/router
byte dnsserver[] = { 192,168,178,1 }; //ip from dns server
byte server[]    = {   0,  0,  0,  0 }; //ip from euse.de (server)

#define HOSTNAME  "www.euse.de"         //host
#define USERAGENT "Bee Hive Scale"     //user agent is the project name
#define INTERVAL 120*1000UL                 //120s/2min delay between updates

unsigned long lastConnectionTime = 0; //last time you connected to the server, in milliseconds

//initialize the client library with the ip and port of the server 
//that you want to connect to (port 80 is default for HTTP)
RedFlyClient client(server, 80);

#include <ADS1231.h>
ADS1231 loadCell;  // create ADS1231 object
long weightSensorValue;
float weightKg;
boolean weightKgTrusted = false;

#include <RunningMedian.h>
// RunningMedian sample size is 11
RunningMedian weightSamples = RunningMedian(11);  // create RunningMedian object

#include <SPI.h>

#include <OneWire.h>
int pin_number = 4; //DS18S20 Signal pin on digital 4
OneWire ds(pin_number);
float temp;

long vcc;

//debug output functions (9600 Baud, 8N2)
//Leonardo boards use USB for communication, so we dont need to disable the RedFly
void debugout(char *s)
{
#if defined(__AVR_ATmega32U4__)
  Serial.print(s);
#else
  RedFly.disable();
  Serial.print(s);
  RedFly.enable();
#endif
}

void debugoutln(char *s)
{
#if defined(__AVR_ATmega32U4__)
  Serial.println(s);
#else
  RedFly.disable();
  Serial.println(s);
  RedFly.enable();
#endif
}

void setup()
{
  uint8_t ret;

#if defined(__AVR_ATmega32U4__) //Leonardo boards use USB for communication
  Serial.begin(9600); //init serial port and set baudrate
  while(!Serial); //wait for serial port to connect (needed for Leonardo only)
#endif

  //init the WiFi module on the shield
  // ret = RedFly.init(br, pwr) //br=9600|19200|38400|57600|115200|200000|230400, pwr=LOW_POWER|MED_POWER|HIGH_POWER
  // ret = RedFly.init(pwr) //9600 baud, pwr=LOW_POWER|MED_POWER|HIGH_POWER
  // ret = RedFly.init() //9600 baud, HIGH_POWER
  ret = RedFly.init();
  if(ret)
  {
    debugoutln("INIT ERR"); //there are problems with the communication between the Arduino and the RedFly
  }
  else
  {
    //scan for wireless networks (must be run before join command)
    RedFly.scan();

    //join network
    // ret = join("wlan-ssid", "wlan-passw", INFRASTRUCTURE or IBSS_JOINER or IBSS_CREATOR, chn, authmode) //join infrastructure or ad-hoc network, or create ad-hoc network
    // ret = join("wlan-ssid", "wlan-passw", IBSS_CREATOR, chn) //create ad-hoc network with password, channel 1-14
    // ret = join("wlan-ssid", IBSS_CREATOR, chn) //create ad-hoc network, channel 1-14
    // ret = join("wlan-ssid", "wlan-passw", INFRASTRUCTURE or IBSS_JOINER) //join infrastructure or ad-hoc network with password
    // ret = join("wlan-ssid", INFRASTRUCTURE or IBSS_JOINER) //join infrastructure or ad-hoc network
    // ret = join("wlan-ssid", "wlan-passw") //join infrastructure network with password
    // ret = join("wlan-ssid") //join infrastructure network
    ret = RedFly.join("[SSID]", "[wlan-passwort]", INFRASTRUCTURE); //die eckigen klammern durch die entsprechenden zugangsinfos ersetzen
    if(ret)
    {
      debugoutln("JOIN ERR");
      for(;;); //do nothing forevermore
    }
    else
    {
      //set ip config
      // ret = RedFly.begin(); //DHCP
      // ret = RedFly.begin(1 or 2); //1=DHCP or 2=Auto-IP
      // ret = RedFly.begin(ip);
      // ret = RedFly.begin(ip, dnsserver);
      // ret = RedFly.begin(ip, dnsserver, gateway);
      // ret = RedFly.begin(ip, dnsserver, gateway, netmask);
      ret = RedFly.begin();
      if(ret)
      {
        debugoutln("BEGIN ERR");
        RedFly.disconnect();
        for(;;); //do nothing forevermore
      }
      else
      {
        if(RedFly.getip(HOSTNAME, server) == 0) //get ip
        {
          //everything okay
          debugoutln("Start sending...");
        }
        else
        {
          debugoutln("DNS ERR");
          RedFly.disconnect();
          for(;;); //do nothing forevermore
        }
      }
    }
  }
   // load cell / ADS1231 pin definition: SCL 8, Data 5, PowerDown 9
  loadCell.attach(8,5,9);
}

void loop()
{
  unsigned long ms = millis();

  if(!client.connected() && ((ms-lastConnectionTime) > INTERVAL))
  {
    lastConnectionTime = ms;

    //read and pepare sensor data
//    RedFly.disable();
    temp = getTemperature();
    vcc = readVcc();
//    sprintf(tmp, "kiste1,%i\r\ntemp,%i\r\nvcc,%i\r\n", kiste1, temp, vcc);
        
    // ADS1231 ready?
    if (loadCell.check()) 
    {
      // delay(1000);
      // read input of ADS1231:
      weightSensorValue = loadCell.readData();
      // calculate weight in kg
      weightKg = ((float)weightSensorValue - loadCellZeroOffset) / loadCellKgDivider;
      // add data to runnig median sample
      weightSamples.add(weightKg);
      // complete sample? 
      if (weightSamples.getCount() == weightSamples.getSize()) 
      {
        weightKgTrusted = true;
      }
    }
//    RedFly.enable();

    //connect to server
    if(client.connect(server, 80)) 
    {
      //send HTTP header
      Serial.println("Verbunden...sende Daten...");
      client.print_P(PSTR("GET /honig/beescale/add_line2.php?kiste1="));
      client.print(weightKg, 3);
      client.print_P(PSTR("&temp="));
      client.print(temp, 1);
      client.print_P(PSTR("&vcc="));
      client.print(vcc);
      client.print_P(PSTR(" HTTP/1.0\r\n"));
      client.print_P(PSTR("User-Agent: "USERAGENT"\r\n"));
      client.print_P(PSTR("Content-Type: text/csv\r\n"));
      client.print_P(PSTR("Connection: close\r\n\r\n"));
      //close connection
      client.stop();
    }
    else
    {
      debugoutln("CONN ERR");
    }
  }
}

#define DS18S20_ID 0x10
#define DS18B20_ID 0x28
 boolean getTemperature(){
 byte i;
 byte present = 0;
 byte data[12];
 byte addr[8];
 //find a device
 if (!ds.search(addr)) {
 ds.reset_search();
 return false;
 }
 if (OneWire::crc8( addr, 7) != addr[7]) {
 return false;
 }
 if (addr[0] != DS18S20_ID && addr[0] != DS18B20_ID) {
 return false;
 }
 ds.reset();
 ds.select(addr);
 // Start conversion
 ds.write(0x44, 1);
 // Wait some time...
 delay(850);
 present = ds.reset();
 ds.select(addr);
 // Issue Read scratchpad command
 ds.write(0xBE);
 // Receive 9 bytes
 for ( i = 0; i < 9; i++) {
 data[i] = ds.read();
 }
 // Calculate temperature value
 temp = ( (data[1] << 8) + data[0] )*0.0625;
 return temp;
 }
 
 long readVcc() {
  // Read 1.1V reference against AVcc
  // set the reference to Vcc and the measurement to the internal 1.1V reference
  #if defined(__AVR_ATmega32U4__) || defined(__AVR_ATmega1280__) || defined(__AVR_ATmega2560__)
    ADMUX = _BV(REFS0) | _BV(MUX4) | _BV(MUX3) | _BV(MUX2) | _BV(MUX1);
  #elif defined (__AVR_ATtiny24__) || defined(__AVR_ATtiny44__) || defined(__AVR_ATtiny84__)
    ADMUX = _BV(MUX5) | _BV(MUX0);
  #elif defined (__AVR_ATtiny25__) || defined(__AVR_ATtiny45__) || defined(__AVR_ATtiny85__)
    ADMUX = _BV(MUX3) | _BV(MUX2);
  #else
    ADMUX = _BV(REFS0) | _BV(MUX3) | _BV(MUX2) | _BV(MUX1);
  #endif  
  delay(2); // Wait for Vref to settle
  ADCSRA |= _BV(ADSC); // Start conversion
  while (bit_is_set(ADCSRA,ADSC)); // measuring
  uint8_t low  = ADCL; // must read ADCL first - it then locks ADCH  
  uint8_t high = ADCH; // unlocks both
  long result = (high<<8) | low;
  result = 1125300L / result; // Calculate Vcc (in mV); 1125300 = 1.1*1023*1000
  return result; // Vcc in millivolts
}
